void sja1000_irq_handler(int irq, void *dev_id, struct pt_regs *regs)
{
	chip=(struct chip_t *)dev_id;
	device=(struct candevice_t *)chip->hostdevice;

	put_reg=device->hwspecops->write_register;
	get_reg=device->hwspecops->read_register;

	irq_register=get_reg(chip->chip_base_addr+SJAIR);
//	DEBUGMSG("sja1000_irq_handler: SJAIR:%02x\n",irq_register);
//	DEBUGMSG("sja1000_irq_handler: SJASR:%02x\n",
//					get_reg(chip->chip_base_addr+SJASR));

	if ((irq_register & (IR_WUI|IR_DOI|IR_EI|IR_TI|IR_RI)) == 0)
		return;

	fifo=chip->msgobj[0]->fifo;
	msgbase=chip->msgobj[0]->obj_base_addr;

	if ((irq_register & IR_RI) != 0) 
		sja1000_irq_read_handler();
	if ((irq_register & IR_TI) != 0) 
		sja1000_irq_write_handler();
	if ((irq_register & (IR_EI|IR_DOI)) != 0) { 
		// Some error happened
// FIXME: chip should be brought to usable state. Transmission cancelled if in progress.
// Reset flag set to 0 if chip is already off the bus. Full state report
		CANMSG("Error: status register: 0x%x irq_register: 0x%02x\n",
			get_reg(chip->chip_base_addr+SJASR), irq_register);
		chip->msgobj[0]->ret=-1;
		if (fifo->writeq)
			wake_up_interruptible(&fifo->writeq);
		if (fifo->readq)
			wake_up_interruptible(&fifo->readq);
	}

	return;
}
void sja1000_irq_read_handler(void)
{
	int i=0, id=0, tmp=1;

	while (tmp) {
		id=get_reg(msgbase+SJARXID0) | (get_reg(msgbase+SJARXID1)<<8);
		fifo->buf_entry[fifo->head].length= id & 0x0f;
		fifo->buf_entry[fifo->head].flags = id&ID0_RTR ? 
			MSG_RTR : 0;
		fifo->buf_entry[fifo->head].timestamp = 0;
		fifo->buf_entry[fifo->head].cob = 0;
		fifo->buf_entry[fifo->head].id=id>>5;
		for (i=0; i<fifo->buf_entry[fifo->head].length; i++)
			fifo->buf_entry[fifo->head].data[i]=get_reg(msgbase +
								SJARXDAT0 + i);

		put_reg(CMR_RRB,msgbase+SJACMR);

		fifo->head++;
		if (fifo->head >= MAX_BUF_LENGTH-1)
			fifo->head=0;

		tmp=(get_reg(msgbase+SJASR) & SR_RBS);
	}
	if (fifo->readq)
		wake_up_interruptible(&fifo->readq);
}

void sja1000_irq_write_handler(void)
{
	if (fifo->writeq)
		wake_up_interruptible(&fifo->writeq);
}
